%Supply your path to the input of tumor read count table with first column 
%showing gene IDs.
male_count=importdata('/Users/bos/repos/kidney-sex/data/expressed_gene_read_count_male.csv');
%number of tumor samples
n1=size(male_count.data,2);

%Supply your path to the input of normal read count table 
female_count=importdata('/Users/bos/repos/kidney-sex/data/expressed_gene_read_count_female.csv');
%number of normal samples
n2=size(female_count.data,2);
%combine read count tables together. 
readcount_table=[male_count.data female_count.data];
gene_name=male_count.rowheaders;
%or you can put both groups of read counts in one file and run the
%following commmand:
% readcount_table=importdata('Both_COUNT_TABLE');
% gene_name=readcount_table.rowheaders;
% readcount_table=readcount_table.data]; 
% n1=# of samples in Group 1
% n2=# of samples in Group 2


%intialization of values and settings that will be used in the fmincon function
x0=[1/n1*ones(n1,1);1/n2*ones(n2,1)];
lb=zeros(n1+n2,1);
ub=ones(n1+n2,1);
beq=[1;1;0];
options = optimoptions(@fmincon,'Algorithm','interior-point','GradObj','on','GradConstr','on','Display','off');

%total number of genes.
N=size(readcount_table,1);
%ELTSeq statistic initialization
ELTSeq_sta=zeros(N,1);

%normalization
nfactor=mean(mean(readcount_table))./mean(readcount_table);
for n=1:N
    readcount_table(n,:)=readcount_table(n,:).*nfactor;
end

%run ELTSeq
perc=10;
for n=1:N
    if n/N>=perc/100
        fprintf('%d%% done\n',perc);
        perc=perc+10;
    end
    %filter the lowly expressed genes
    if median(readcount_table(n,1:n1))==0 && median(readcount_table(n,n1+1:n1+n2))==0
        ELTSeq_sta(n)=-Inf;
        continue;
    end
    %deal with extreme case
    if min(readcount_table(n,1:n1))>=max(readcount_table(n,n1+1:n1+n2)) || max(readcount_table(n,1:n1))<=min(readcount_table(n,n1+1:n1+n2))
        ELTSeq_sta(n)=inf;
    else
        %use fmincon function to get test statistics of ELTSeq
        f=@(x)assign(x,readcount_table(n,1:n1),readcount_table(n,n1+1:n1+n2));
        Aeq=[ones(1,n1) zeros(1,n2);zeros(1,n1) ones(1,n2);readcount_table(n,1:n1) -readcount_table(n,n1+1:n1+n2)];
        [xmin,ELTSeq_sta(n)] = fmincon(@negloglike,x0,[],[],Aeq,beq,lb,ub,f,options);
    end
end
%-2log(R) is the final ELTSeq test statistic, which has an asymptotic
%distuibution of Chi-square with d.f. 1.
ELTSeq_sta=2*(ELTSeq_sta-n1*log(n1)-n2*log(n2));
[stat,index]=sort(ELTSeq_sta,'descend');
digits(5);
%calculate asymptotic p-values
sortp=1-chi2cdf(stat,1);
%calculate BH-FDR
fdr=sortp*N./(1:N)';
fdr(fdr>1)=1;
st=1;
curmin=fdr(1);
for idx=2:N
    if fdr(idx)>curmin
        fdr(st:idx-1)=curmin;
        st=idx;
    end
    curmin=fdr(idx);
end
fdr(st:idx)=curmin;

%save ELTSeq DE analysis results in a file named 'DE_res.txt'.
res=cell(1,2);
res{2}=[stat sortp fdr];
res{1}=gene_name(index);
fileID = fopen('DE_res.txt','w');
fprintf(fileID,'%10s %10s %10s %10s\n','Gene_name','ELTSeq_stat','p-value','FDR');
formatSpec = '%10s %10.8f %10.5e %10.5e\n';
for row = 1:N
    fprintf(fileID,formatSpec,res{1}{row,:},res{2}(row,:));
end
fclose(fileID);